@@ -1446,7 +1446,7 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 
 	e_table->use_click_to_add = specification->click_to_add;
 	e_table->use_click_to_add_end = specification->click_to_add_end;
-	e_table->click_to_add_message = e_utf8_from_locale_string (dgettext (e_table->domain, specification->click_to_add_message));
+	e_table->click_to_add_message = g_strdup (dgettext (e_table->domain, specification->click_to_add_message));
 	e_table->alternating_row_colors = specification->alternating_row_colors;
 	e_table->horizontal_draw_grid = specification->horizontal_draw_grid;
 	e_table->vertical_draw_grid = specification->vertical_draw_grid;
@@ -2765,9 +2765,6 @@ do_drag_motion(ETable *et,
 
 	widget = GTK_WIDGET (et);
 
-	x -= widget->allocation.x;
-	y -= widget->allocation.y;
-
 	e_table_get_cell_at (et, x, y, &row, &col);
 
 	if (row != et->drop_row && col != et->drop_row) {
@@ -2906,9 +2903,6 @@ et_drag_motion(GtkWidget *widget,
 				  time);
 
 
-	x -= widget->allocation.x;
-	y -= widget->allocation.y;
-
 	if (y < 20)
 		direction |= ET_SCROLL_UP;
 	if (y > widget->allocation.height - 20)
@@ -2937,9 +2931,6 @@ et_drag_drop(GtkWidget *widget,
 	gboolean ret_val;
 	int row, col;
 
-	x -= widget->allocation.x;
-	y -= widget->allocation.y;
-
 	e_table_get_cell_at (et, x, y, &row, &col);
 
 	if (row != et->drop_row && col != et->drop_row) {
@@ -2972,9 +2963,6 @@ et_drag_data_received(GtkWidget *widget,
 {
 	int row, col;
 
-	x -= widget->allocation.x;
-	y -= widget->allocation.y;
-
 	e_table_get_cell_at (et, x, y, &row, &col);
 
 	g_signal_emit (G_OBJECT (et), et_signals [TABLE_DRAG_DATA_RECEIVED], 0,
