@@ -576,7 +576,7 @@ e_icon_bar_style_set (GtkWidget *widget, GtkStyle *previous_style) {
 				       EIconBarItem, item_num);
 
 		flattened = flatten_alpha (item->pixbuf,
-					   rgb_from_gdk_color (&style->bg [GTK_STATE_NORMAL]));
+					   rgb_from_gdk_color (&style->bg [GTK_STATE_ACTIVE]));
 
 		gnome_canvas_item_set (item->image, 				      
 				       "GnomeCanvasPixbuf::pixbuf", flattened ? flattened : item->pixbuf,
@@ -590,7 +590,7 @@ e_icon_bar_style_set (GtkWidget *widget, GtkStyle *previous_style) {
 			gdk_pixbuf_unref (flattened);
 
 	}
-	
+
 	e_icon_bar_recalc_item_positions (icon_bar);
 }
 
@@ -668,7 +668,7 @@ e_icon_bar_add_item (EIconBar	    *icon_bar,
 		gdk_pixbuf_ref (image);
 
 	flattened = flatten_alpha (image,
-				   rgb_from_gdk_color (&style->bg [GTK_STATE_NORMAL]));
+				   rgb_from_gdk_color (&style->bg [GTK_STATE_ACTIVE]));
 	
 	item.image = gnome_canvas_item_new (GNOME_CANVAS_GROUP (GNOME_CANVAS (icon_bar)->root),
 					    gnome_canvas_pixbuf_get_type (),
@@ -825,7 +825,7 @@ e_icon_bar_set_item_image	(EIconBar	  *icon_bar,
 	item = &g_array_index (icon_bar->items, EIconBarItem, item_num);
 
 	flattened = flatten_alpha (item->pixbuf,
-				   rgb_from_gdk_color (&style->bg [GTK_STATE_NORMAL]));
+				   rgb_from_gdk_color (&style->bg [GTK_STATE_ACTIVE]));
 
 	gnome_canvas_item_set (item->image,
 			       "GnomeCanvasPixbuf::pixbuf", flattened ? flattened : image,
