@@ -5,7 +5,7 @@
  *
  * Authors:
  *   Michael Zucchi <notzed@ximian.com>
- *   Jeffery Steadfast <fejj@ximian.com>
+ *   Jeffery Stedfast <fejj@ximian.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,24 +28,18 @@
 
 #include <string.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 #include <glib.h>
 #include "e-iconv.h"
 
 #include <locale.h>
 
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
-
 #ifdef HAVE_CODESET
 #include <langinfo.h>
 #endif
 
-/* FIXME: Use feature test */
-#ifndef __sun__
-#define ICONV_ISO_NEEDS_DASH (1)
-#endif
+#include "iconv-detect.h"
 
 #define cd(x) 
 
@@ -98,7 +92,8 @@ static GHashTable *iconv_cache_open;
 static unsigned int iconv_cache_size = 0;
 
 static GHashTable *iconv_charsets = NULL;
-static const char *locale_charset = NULL;
+static char *locale_charset = NULL;
+static char *locale_lang = NULL;
 
 struct {
 	char *charset;
@@ -137,6 +132,7 @@ struct {
 
 	/* FIXME: Japanese/Korean/Chinese stuff needs checking */
 	{ "euckr-0",        "EUC-KR"     },
+	{ "5601",           "EUC-KR"     },
 	{ "big5-0",         "BIG5"       },
 	{ "big5.eten-0",    "BIG5"       },
 	{ "big5hkscs-0",    "BIG5HKCS"   },
@@ -149,6 +145,7 @@ struct {
 	{ "ujis-0",         "ujis"  	 },
 	{ "jisx0208.1983-0","SJIS"       },
 	{ "jisx0212.1990-0","SJIS"       },
+	{ "pck",	    "SJIS"       },
 	{ NULL,             NULL         }
 };
 
@@ -189,6 +186,44 @@ static EDListNode *e_dlist_remove(EDListNode *n)
         return n;
 }
 
+static void
+locale_parse_lang (const char *locale)
+{
+	char *codeset, *lang;
+	
+	if ((codeset = strchr (locale, '.')))
+		lang = g_strndup (locale, codeset - locale);
+	else
+		lang = g_strdup (locale);
+	
+	/* validate the language */
+	if (strlen (lang) >= 2) {
+		if (lang[2] == '-' || lang[2] == '_') {
+			/* canonicalise the lang */
+			g_ascii_strdown (lang, 2);
+			
+			/* validate the country code */
+			if (strlen (lang + 3) > 2) {
+				/* invalid country code */
+				lang[2] = '\0';
+			} else {
+				lang[2] = '-';
+				g_ascii_strup (lang + 3, 2);
+			}
+		} else if (lang[2] != '\0') {
+			/* invalid language */
+			g_free (lang);
+			lang = NULL;
+		}
+		
+		locale_lang = lang;
+	} else {
+		/* invalid language */
+		locale_lang = NULL;
+		g_free (lang);
+	}
+}
+
 /* NOTE: Owns the lock on return if keep is TRUE ! */
 static void
 e_iconv_init(int keep)
@@ -209,7 +244,7 @@ e_iconv_init(int keep)
 	for (i = 0; known_iconv_charsets[i].charset != NULL; i++) {
 		from = g_strdup(known_iconv_charsets[i].charset);
 		to = g_strdup(known_iconv_charsets[i].iconv_name);
-		g_strdown(from);
+		g_ascii_strdown (from, -1);
 		g_hash_table_insert(iconv_charsets, from, to);
 	}
 
@@ -218,7 +253,7 @@ e_iconv_init(int keep)
 	iconv_cache_open = g_hash_table_new(NULL, NULL);
 	
 	locale = setlocale (LC_ALL, NULL);
-
+	
 	if (!locale || !strcmp (locale, "C") || !strcmp (locale, "POSIX")) {
 		/* The locale "C"  or  "POSIX"  is  a  portable  locale;  its
 		 * LC_CTYPE  part  corresponds  to  the 7-bit ASCII character
@@ -226,10 +261,11 @@ e_iconv_init(int keep)
 		 */
 		
 		locale_charset = NULL;
+		locale_lang = NULL;
 	} else {
 #ifdef HAVE_CODESET
-		locale_charset = g_strdup(nl_langinfo(CODESET));
-		g_strdown((char *)locale_charset);
+		locale_charset = g_strdup (nl_langinfo (CODESET));
+		g_ascii_strdown (locale_charset, -1);
 #else
 		/* A locale name is typically of  the  form  language[_terri-
 		 * tory][.codeset][@modifier],  where  language is an ISO 639
@@ -246,12 +282,16 @@ e_iconv_init(int keep)
 			/* ; is a hack for debian systems and / is a hack for Solaris systems */
 			for (p = codeset; *p && !strchr ("@;/", *p); p++);
 			locale_charset = g_strndup (codeset, p - codeset);
-			g_strdown (locale_charset);
+			g_ascii_strdown (locale_charset, -1);
 		} else {
 			/* charset unknown */
 			locale_charset = NULL;
 		}
-#endif
+#endif		
+		
+		/* parse the locale lang */
+		locale_parse_lang (locale);
+
 	}
 
 	if (!keep)
@@ -265,30 +305,52 @@ const char *e_iconv_charset_name(const char *charset)
 	if (charset == NULL)
 		return NULL;
 
-	name = alloca(strlen(charset)+1);
-	strcpy(name, charset);
-	g_strdown(name);
-
+	name = g_alloca (strlen (charset) + 1);
+	strcpy (name, charset);
+	g_ascii_strdown (name, -1);
+	
 	e_iconv_init(TRUE);
 	ret = g_hash_table_lookup(iconv_charsets, name);
 	if (ret != NULL) {
 		UNLOCK();
 		return ret;
-
-
 	}
 
 	/* Unknown, try canonicalise some basic charset types to something that should work */
 	if (strncmp(name, "iso", 3) == 0) {
 		/* Convert iso-nnnn-n or isonnnn-n or iso_nnnn-n to iso-nnnn-n or isonnnn-n */
-		tmp = name+3;
-		if (tmp[0] == '_' || tmp[0] == '-')
+		int iso, codepage;
+		char *p;
+		
+		tmp = name + 3;
+		if (*tmp == '-' || *tmp == '_')
 			tmp++;
-#ifdef ICONV_ISO_NEEDS_DASH
-		ret = g_strdup_printf("ISO-%s", tmp);
-#else
-		ret = g_strdup_printf("ISO%s", tmp);
-#endif
+		
+		iso = strtoul (tmp, &p, 10);
+		
+		if (iso == 10646) {
+			/* they all become ICONV_10646 */
+			ret = g_strdup (ICONV_10646);
+		} else {
+			tmp = p;
+			if (*tmp == '-' || *tmp == '_')
+				tmp++;
+			
+			codepage = strtoul (tmp, &p, 10);
+			
+			if (p > tmp) {
+				/* codepage is numeric */
+#ifdef __aix__
+				if (codepage == 13)
+					ret = g_strdup ("IBM-921");
+				else
+#endif /* __aix__ */
+					ret = g_strdup_printf (ICONV_ISO_D_FORMAT, iso, codepage);
+			} else {
+				/* codepage is a string - probably iso-2022-jp or something */
+				ret = g_strdup_printf (ICONV_ISO_S_FORMAT, iso, p);
+			}
+		}
 	} else if (strncmp(name, "windows-", 8) == 0) {
 		/* Convert windows-nnnnn or windows-cpnnnnn to cpnnnn */
 		tmp = name+8;
@@ -346,7 +408,7 @@ iconv_t e_iconv_open(const char *oto, const char *ofrom)
 
 	to = e_iconv_charset_name (oto);
 	from = e_iconv_charset_name (ofrom);
-	tofrom = alloca(strlen(to) +strlen(from) + 2);
+	tofrom = g_alloca (strlen (to) + strlen (from) + 2);
 	sprintf(tofrom, "%s%%%s", to, from);
 
 	LOCK();
@@ -392,10 +454,11 @@ iconv_t e_iconv_open(const char *oto, const char *ofrom)
 			/* work around some broken iconv implementations 
 			 * that die if the length arguments are NULL 
 			 */
-			size_t buggy_iconv = 0;
+			size_t buggy_iconv_len = 0;
+			gchar *buggy_iconv_buf = NULL;
 
 			/* resets the converter */
-			iconv(ip, NULL, &buggy_iconv, NULL, &buggy_iconv);
+			iconv(ip, &buggy_iconv_buf, &buggy_iconv_len, &buggy_iconv_buf, &buggy_iconv_len);
 			in->busy = TRUE;
 			e_dlist_remove((EDListNode *)in);
 			e_dlist_addhead(&ic->open, (EDListNode *)in);
@@ -456,3 +519,54 @@ const char *e_iconv_locale_charset(void)
 	return locale_charset;
 }
 
+
+const char *
+e_iconv_locale_language (void)
+{
+	e_iconv_init (FALSE);
+	
+	return locale_lang;
+}
+
+/* map CJKR charsets to their language code */
+/* NOTE: only support charset names that will be returned by
+ * e_iconv_charset_name() so that we don't have to keep track of all
+ * the aliases too. */
+static struct {
+	char *charset;
+	char *lang;
+} cjkr_lang_map[] = {
+	{ "Big5",        "zh" },
+	{ "BIG5HKCS",    "zh" },
+	{ "gb2312",      "zh" },
+	{ "gb18030",     "zh" },
+	{ "gbk",         "zh" },
+	{ "euc-tw",      "zh" },
+	{ "iso-2022-jp", "ja" },
+	{ "sjis",        "ja" },
+	{ "ujis",        "ja" },
+	{ "eucJP",       "ja" },
+	{ "euc-jp",      "ja" },
+	{ "euc-kr",      "ko" },
+	{ "koi8-r",      "ru" },
+	{ "koi8-u",      "uk" }
+};
+
+#define NUM_CJKR_LANGS (sizeof (cjkr_lang_map) / sizeof (cjkr_lang_map[0]))
+
+const char *
+e_iconv_charset_language (const char *charset)
+{
+	int i;
+	
+	if (!charset)
+		return NULL;
+	
+	charset = e_iconv_charset_name (charset);
+	for (i = 0; i < NUM_CJKR_LANGS; i++) {
+		if (!strcasecmp (cjkr_lang_map[i].charset, charset))
+			return cjkr_lang_map[i].lang;
+	}
+	
+	return NULL;
+}
