@@ -128,6 +128,10 @@ e_canvas_dispose (GObject *object)
 		g_source_remove(canvas->idle_id);
 	canvas->idle_id = 0;
 
+	if (canvas->grab_cancelled_check_id)
+		g_source_remove (canvas->grab_cancelled_check_id);
+	canvas->grab_cancelled_check_id = 0;
+
 	if (canvas->toplevel) {
 		if (canvas->visibility_notify_id)
 			g_signal_handler_disconnect (canvas->toplevel,
@@ -157,7 +161,7 @@ e_canvas_new ()
 static int
 emit_event (GnomeCanvas *canvas, GdkEvent *event)
 {
-	GdkEvent ev;
+	GdkEvent *ev;
 	gint finished;
 	GnomeCanvasItem *item;
 	GnomeCanvasItem *parent;
@@ -221,14 +225,14 @@ emit_event (GnomeCanvas *canvas, GdkEvent *event)
 	 * offsets of the fields in the event structures.
 	 */
 
-	ev = *event;
+	ev = gdk_event_copy (event);
 
-	switch (ev.type) {
+	switch (ev->type) {
 	case GDK_ENTER_NOTIFY:
 	case GDK_LEAVE_NOTIFY:
 		gnome_canvas_window_to_world (canvas,
-					      ev.crossing.x, ev.crossing.y,
-					      &ev.crossing.x, &ev.crossing.y);
+					      ev->crossing.x, ev->crossing.y,
+					      &ev->crossing.x, &ev->crossing.y);
 		break;
 
 	case GDK_MOTION_NOTIFY:
@@ -237,8 +241,8 @@ emit_event (GnomeCanvas *canvas, GdkEvent *event)
 	case GDK_3BUTTON_PRESS:
 	case GDK_BUTTON_RELEASE:
 		gnome_canvas_window_to_world (canvas,
-					      ev.motion.x, ev.motion.y,
-					      &ev.motion.x, &ev.motion.y);
+					      ev->motion.x, ev->motion.y,
+					      &ev->motion.x, &ev->motion.y);
 		break;
 
 	default:
@@ -255,17 +259,7 @@ emit_event (GnomeCanvas *canvas, GdkEvent *event)
 	while (item && !finished) {
 		g_object_ref (item);
 
-		g_signal_emit_by_name (item, "event",
-				       &ev,
-				       &finished);
-
-#ifndef NO_WARNINGS
-#warning FIXME - needs thought
-#endif
-#if 0
-		if (GTK_OBJECT_DESTROYED (item))
-			finished = TRUE;
-#endif
+		g_signal_emit_by_name (item, "event", ev, &finished);
 
 		parent = item->parent;
 		g_object_unref (item);
@@ -273,6 +267,8 @@ emit_event (GnomeCanvas *canvas, GdkEvent *event)
 		item = parent;
 	}
 
+	gdk_event_free (ev);
+
 	return finished;
 }
 
@@ -692,8 +688,8 @@ e_canvas_style_set (GtkWidget *widget, GtkStyle *previous_style)
 static void
 e_canvas_realize (GtkWidget *widget)
 {
-	gint width, height;
 #ifdef GAL_GDK_IM
+	gint width, height;
 	ECanvas *ecanvas = E_CANVAS (widget);
 #endif
 
