@@ -1,38 +1,41 @@
-/* GTK - The GIMP Toolkit
- * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* 
+ * e-scroll-frame.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * Authors:
+ *   Federico Mena <federico@ximian.com>
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * EScrollFrame based on GtkScrolledWindow.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
-
-/*
  * Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
- */
-
-/* EScrollFrame based on GtkScrolledWindow.
  *
- * Modifications by Federico Mena <federico@helixcode.com>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
 
+#include <gal/util/e-util.h>
+#include <gal/util/e-i18n.h>
+
 #include <gtk/gtksignal.h>
 #include <gtk/gtkviewport.h>
+#include <gtk/gtktypebuiltins.h>
 
 #include "e-vscrollbar.h"
 #include "e-hscrollbar.h"
@@ -77,14 +80,14 @@
 
 /* Object argument IDs */
 enum {
-	ARG_0,
-	ARG_HADJUSTMENT,
-	ARG_VADJUSTMENT,
-	ARG_HSCROLLBAR_POLICY,
-	ARG_VSCROLLBAR_POLICY,
-	ARG_FRAME_PLACEMENT,
-	ARG_SHADOW_TYPE,
-	ARG_SCROLLBAR_SPACING
+	PROP_0,
+	PROP_HADJUSTMENT,
+	PROP_VADJUSTMENT,
+	PROP_HSCROLLBAR_POLICY,
+	PROP_VSCROLLBAR_POLICY,
+	PROP_FRAME_PLACEMENT,
+	PROP_SHADOW_TYPE,
+	PROP_SCROLLBAR_SPACING
 };
 
 /* Private part of the EScrollFrame structure */
@@ -120,14 +123,13 @@ typedef struct {
 
 static void e_scroll_frame_class_init (EScrollFrameClass *class);
 static void e_scroll_frame_init (EScrollFrame *sf);
-static void e_scroll_frame_set_arg (GtkObject *object, GtkArg *arg, guint arg_id);
-static void e_scroll_frame_get_arg (GtkObject *object, GtkArg *arg, guint arg_id);
-static void e_scroll_frame_destroy (GtkObject *object);
-static void e_scroll_frame_finalize (GtkObject *object);
+static void e_scroll_frame_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
+static void e_scroll_frame_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
+static void e_scroll_frame_dispose (GObject *object);
+static void e_scroll_frame_finalize (GObject *object);
 
 static void e_scroll_frame_map (GtkWidget *widget);
 static void e_scroll_frame_unmap (GtkWidget *widget);
-static void e_scroll_frame_draw (GtkWidget *widget, GdkRectangle *area);
 static void e_scroll_frame_size_request (GtkWidget *widget, GtkRequisition *requisition);
 static void e_scroll_frame_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
 static gint e_scroll_frame_expose (GtkWidget *widget, GdkEventExpose *event);
@@ -137,6 +139,7 @@ static void e_scroll_frame_remove (GtkContainer *container, GtkWidget *widget);
 static void e_scroll_frame_forall (GtkContainer *container, gboolean include_internals,
 				     GtkCallback callback, gpointer callback_data);
 
+#define PARENT_TYPE GTK_TYPE_BIN
 static GtkBinClass *parent_class;
 
 
@@ -149,80 +152,34 @@ static GtkBinClass *parent_class;
  *
  * Return value: The type ID of the #EScrollFrame class.
  **/
-GtkType
-e_scroll_frame_get_type (void)
-{
-	static GtkType scroll_frame_type = 0;
-
-	if (!scroll_frame_type) {
-		static const GtkTypeInfo scroll_frame_info = {
-			"EScrollFrame",
-			sizeof (EScrollFrame),
-			sizeof (EScrollFrameClass),
-			(GtkClassInitFunc) e_scroll_frame_class_init,
-			(GtkObjectInitFunc) e_scroll_frame_init,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		scroll_frame_type = gtk_type_unique (GTK_TYPE_BIN, &scroll_frame_info);
-	}
-
-	return scroll_frame_type;
-}
+E_MAKE_TYPE (e_scroll_frame,
+	     "EScrollFrame",
+	     EScrollFrame,
+	     e_scroll_frame_class_init,
+	     e_scroll_frame_init,
+	     PARENT_TYPE)
 
 /* Class initialization function for the scroll frame widget */
 static void
 e_scroll_frame_class_init (EScrollFrameClass *class)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	GtkWidgetClass *widget_class;
 	GtkContainerClass *container_class;
 
-	object_class = (GtkObjectClass *) class;
+	object_class = (GObjectClass *) class;
 	widget_class = (GtkWidgetClass *) class;
 	container_class = (GtkContainerClass *) class;
 
-	parent_class = gtk_type_class (GTK_TYPE_BIN);
+	parent_class = g_type_class_ref (PARENT_TYPE);
 
-	gtk_object_add_arg_type ("EScrollFrame::hadjustment",
-				 GTK_TYPE_ADJUSTMENT,
-				 GTK_ARG_READWRITE | GTK_ARG_CONSTRUCT,
-				 ARG_HADJUSTMENT);
-	gtk_object_add_arg_type ("EScrollFrame::vadjustment",
-				 GTK_TYPE_ADJUSTMENT,
-				 GTK_ARG_READWRITE | GTK_ARG_CONSTRUCT,
-				 ARG_VADJUSTMENT);
-	gtk_object_add_arg_type ("EScrollFrame::hscrollbar_policy",
-				 GTK_TYPE_POLICY_TYPE,
-				 GTK_ARG_READWRITE,
-				 ARG_HSCROLLBAR_POLICY);
-	gtk_object_add_arg_type ("EScrollFrame::vscrollbar_policy",
-				 GTK_TYPE_POLICY_TYPE,
-				 GTK_ARG_READWRITE,
-				 ARG_VSCROLLBAR_POLICY);
-	gtk_object_add_arg_type ("EScrollFrame::frame_placement",
-				 GTK_TYPE_CORNER_TYPE,
-				 GTK_ARG_READWRITE,
-				 ARG_FRAME_PLACEMENT);
-	gtk_object_add_arg_type ("EScrollFrame::shadow_type",
-				 GTK_TYPE_SHADOW_TYPE,
-				 GTK_ARG_READWRITE,
-				 ARG_SHADOW_TYPE);
-	gtk_object_add_arg_type ("EScrollFrame::scrollbar_spacing",
-				 GTK_TYPE_UINT,
-				 GTK_ARG_READWRITE,
-				 ARG_SCROLLBAR_SPACING);
-
-	object_class->set_arg = e_scroll_frame_set_arg;
-	object_class->get_arg = e_scroll_frame_get_arg;
-	object_class->destroy = e_scroll_frame_destroy;
+	object_class->set_property = e_scroll_frame_set_property;
+	object_class->get_property = e_scroll_frame_get_property;
+	object_class->dispose = e_scroll_frame_dispose;
 	object_class->finalize = e_scroll_frame_finalize;
 
 	widget_class->map = e_scroll_frame_map;
 	widget_class->unmap = e_scroll_frame_unmap;
-	widget_class->draw = e_scroll_frame_draw;
 	widget_class->size_request = e_scroll_frame_size_request;
 	widget_class->size_allocate = e_scroll_frame_size_allocate;
 	widget_class->expose_event = e_scroll_frame_expose;
@@ -231,6 +188,55 @@ e_scroll_frame_class_init (EScrollFrameClass *class)
 	container_class->add = e_scroll_frame_add;
 	container_class->remove = e_scroll_frame_remove;
 	container_class->forall = e_scroll_frame_forall;
+
+	g_object_class_install_property (object_class, PROP_HADJUSTMENT,
+					 g_param_spec_object ("hadjustment",
+							      _( "HAdjustment" ),
+							      _( "HAdjustment" ),
+							      GTK_TYPE_ADJUSTMENT,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+	g_object_class_install_property (object_class, PROP_VADJUSTMENT,
+					 g_param_spec_object ("vadjustment",
+							      _( "VAdjustment" ),
+							      _( "VAdjustment" ),
+							      GTK_TYPE_ADJUSTMENT,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+	g_object_class_install_property (object_class, PROP_HSCROLLBAR_POLICY,
+					 g_param_spec_enum ("hscrollbar_policy",
+							    _( "hscrollbar policy" ),
+							    _( "hscrollbar policy" ),
+							    GTK_TYPE_POLICY_TYPE, GTK_POLICY_AUTOMATIC,
+							    G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_VSCROLLBAR_POLICY,
+					 g_param_spec_enum ("vscrollbar_policy",
+							    _( "vscrollbar policy" ),
+							    _( "vscrollbar policy" ),
+							    GTK_TYPE_POLICY_TYPE, GTK_POLICY_AUTOMATIC,
+							    G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_FRAME_PLACEMENT,
+					 g_param_spec_enum ("frame_placement",
+							    _( "Frame placement" ),
+							    _( "Frame placement" ),
+							    GTK_TYPE_CORNER_TYPE, GTK_CORNER_TOP_LEFT,
+							    G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_SHADOW_TYPE,
+					 g_param_spec_enum ("shadow_type",
+							    _( "Shadow type" ),
+							    _( "Shadow type" ),
+							    GTK_TYPE_SHADOW_TYPE, GTK_CORNER_TOP_LEFT,
+							    G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_SHADOW_TYPE,
+					 g_param_spec_uint ("scrollbar_spacing",
+							    _( "Scrollbar spacing" ),
+							    _( "Scrollbar spacing" ),
+							    0, G_MAXUINT, 0,
+							    G_PARAM_READWRITE));
 }
 
 /* Object initialization function for the scroll frame widget */
@@ -255,7 +261,7 @@ e_scroll_frame_init (EScrollFrame *sf)
 
 /* Set_arg handler for the scroll frame widget */
 static void
-e_scroll_frame_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+e_scroll_frame_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
 	EScrollFrame *sf;
 	ScrollFramePrivate *priv;
@@ -263,33 +269,33 @@ e_scroll_frame_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	sf = E_SCROLL_FRAME (object);
 	priv = sf->priv;
 
-	switch (arg_id) {
-	case ARG_HADJUSTMENT:
-		e_scroll_frame_set_hadjustment (sf, GTK_VALUE_POINTER (*arg));
+	switch (prop_id) {
+	case PROP_HADJUSTMENT:
+		e_scroll_frame_set_hadjustment (sf, g_value_get_object (value));
 		break;
 
-	case ARG_VADJUSTMENT:
-		e_scroll_frame_set_vadjustment (sf, GTK_VALUE_POINTER (*arg));
+	case PROP_VADJUSTMENT:
+		e_scroll_frame_set_vadjustment (sf, g_value_get_object (value));
 		break;
 
-	case ARG_HSCROLLBAR_POLICY:
-		e_scroll_frame_set_policy (sf, GTK_VALUE_ENUM (*arg), priv->vsb_policy);
+	case PROP_HSCROLLBAR_POLICY:
+		e_scroll_frame_set_policy (sf, g_value_get_enum (value), priv->vsb_policy);
 		break;
 
-	case ARG_VSCROLLBAR_POLICY:
-		e_scroll_frame_set_policy (sf, priv->hsb_policy, GTK_VALUE_ENUM (*arg));
+	case PROP_VSCROLLBAR_POLICY:
+		e_scroll_frame_set_policy (sf, priv->hsb_policy, g_value_get_enum (value));
 		break;
 
-	case ARG_FRAME_PLACEMENT:
-		e_scroll_frame_set_placement (sf, GTK_VALUE_ENUM (*arg));
+	case PROP_FRAME_PLACEMENT:
+		e_scroll_frame_set_placement (sf, g_value_get_enum (value));
 		break;
 
-	case ARG_SHADOW_TYPE:
-		e_scroll_frame_set_shadow_type (sf, GTK_VALUE_ENUM (*arg));
+	case PROP_SHADOW_TYPE:
+		e_scroll_frame_set_shadow_type (sf, g_value_get_enum (value));
 		break;
 
-	case ARG_SCROLLBAR_SPACING:
-		e_scroll_frame_set_scrollbar_spacing (sf, GTK_VALUE_UINT (*arg));
+	case PROP_SCROLLBAR_SPACING:
+		e_scroll_frame_set_scrollbar_spacing (sf, g_value_get_uint (value));
 		break;
 
 	default:
@@ -299,7 +305,7 @@ e_scroll_frame_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 
 /* Get_arg handler for the scroll frame widget */
 static void
-e_scroll_frame_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+e_scroll_frame_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
 	EScrollFrame *sf;
 	ScrollFramePrivate *priv;
@@ -307,44 +313,44 @@ e_scroll_frame_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	sf = E_SCROLL_FRAME (object);
 	priv = sf->priv;
 
-	switch (arg_id) {
-	case ARG_HADJUSTMENT:
-		GTK_VALUE_POINTER (*arg) = e_scroll_frame_get_hadjustment (sf);
+	switch (prop_id) {
+	case PROP_HADJUSTMENT:
+		g_value_set_object (value, e_scroll_frame_get_hadjustment (sf));
 		break;
 
-	case ARG_VADJUSTMENT:
-		GTK_VALUE_POINTER (*arg) = e_scroll_frame_get_vadjustment (sf);
+	case PROP_VADJUSTMENT:
+		g_value_set_object (value, e_scroll_frame_get_vadjustment (sf));
 		break;
 
-	case ARG_HSCROLLBAR_POLICY:
-		GTK_VALUE_ENUM (*arg) = priv->hsb_policy;
+	case PROP_HSCROLLBAR_POLICY:
+		g_value_set_enum (value, priv->hsb_policy);
 		break;
 
-	case ARG_VSCROLLBAR_POLICY:
-		GTK_VALUE_ENUM (*arg) = priv->vsb_policy;
+	case PROP_VSCROLLBAR_POLICY:
+		g_value_set_enum (value, priv->vsb_policy);
 		break;
 
-	case ARG_FRAME_PLACEMENT:
-		GTK_VALUE_ENUM (*arg) = priv->frame_placement;
+	case PROP_FRAME_PLACEMENT:
+		g_value_set_enum (value, priv->frame_placement);
 		break;
 
-	case ARG_SHADOW_TYPE:
-		GTK_VALUE_ENUM (*arg) = priv->shadow_type;
+	case PROP_SHADOW_TYPE:
+		g_value_set_enum (value, priv->shadow_type);
 		break;
 
-	case ARG_SCROLLBAR_SPACING:
-		GTK_VALUE_UINT (*arg) = priv->sb_spacing;
+	case PROP_SCROLLBAR_SPACING:
+		g_value_set_uint (value, priv->sb_spacing);
 		break;
 
 	default:
-		arg->type = GTK_TYPE_INVALID;
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
 /* Destroy handler for the scroll frame widget */
 static void
-e_scroll_frame_destroy (GtkObject *object)
+e_scroll_frame_dispose (GObject *object)
 {
 	EScrollFrame *sf;
 	ScrollFramePrivate *priv;
@@ -355,18 +361,25 @@ e_scroll_frame_destroy (GtkObject *object)
 	sf = E_SCROLL_FRAME (object);
 	priv = sf->priv;
 
-	gtk_widget_unparent (priv->hsb);
-	gtk_widget_unparent (priv->vsb);
-	gtk_widget_destroy (priv->hsb);
-	gtk_widget_destroy (priv->vsb);
+	if (priv->hsb) {
+		gtk_widget_unparent (priv->hsb);
+		gtk_widget_destroy (priv->hsb);
+		priv->hsb = NULL;
+	}
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	if (priv->vsb) {
+		gtk_widget_unparent (priv->vsb);
+		gtk_widget_destroy (priv->vsb);
+		priv->vsb = NULL;
+	}
+
+	if (G_OBJECT_CLASS (parent_class)->dispose)
+		(* G_OBJECT_CLASS (parent_class)->dispose) (object);
 }
 
 /* Finalize handler for the scroll frame widget */
 static void
-e_scroll_frame_finalize (GtkObject *object)
+e_scroll_frame_finalize (GObject *object)
 {
 	EScrollFrame *sf;
 	ScrollFramePrivate *priv;
@@ -374,13 +387,10 @@ e_scroll_frame_finalize (GtkObject *object)
 	sf = E_SCROLL_FRAME (object);
 	priv = sf->priv;
 
-	gtk_widget_unref (priv->hsb);
-	gtk_widget_unref (priv->vsb);
-
 	g_free (priv);
 
-	if (GTK_OBJECT_CLASS (parent_class)->finalize)
-		(* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
+	if (G_OBJECT_CLASS (parent_class)->finalize)
+		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
 /* Map handler for the scroll frame widget */
@@ -450,6 +460,7 @@ draw_shadow (EScrollFrame *sf, GdkRectangle *area)
 			  priv->frame_w, priv->frame_h);
 }
 
+#if 0
 /* Draw handler for the scroll frame widget */
 static void
 e_scroll_frame_draw (GtkWidget *widget, GdkRectangle *area)
@@ -482,6 +493,7 @@ e_scroll_frame_draw (GtkWidget *widget, GdkRectangle *area)
 	    && gtk_widget_intersect (priv->vsb, area, &child_area))
 		gtk_widget_draw (priv->vsb, &child_area);
 }
+#endif
 
 /* Forall handler for the scroll frame widget */
 static void
@@ -540,15 +552,15 @@ e_scroll_frame_size_request (GtkWidget *widget, GtkRequisition *requisition)
 	requisition->height = GTK_CONTAINER (widget)->border_width * 2;
 
 	if (priv->shadow_type != GTK_SHADOW_NONE) {
-		requisition->width += 2 * widget->style->klass->xthickness;
-		requisition->height += 2 * widget->style->klass->ythickness;
+		requisition->width += 2 * widget->style->xthickness;
+		requisition->height += 2 * widget->style->ythickness;
 	}
 
 	gtk_widget_size_request (priv->hsb, &hsb_requisition);
 	gtk_widget_size_request (priv->vsb, &vsb_requisition);
 
 	if (bin->child && GTK_WIDGET_VISIBLE (bin->child)) {
-		static guint quark_aux_info;
+		static GQuark quark_aux_info;
 
 		if (!quark_aux_info)
 			quark_aux_info = g_quark_from_static_string ("gtk-aux-info");
@@ -560,8 +572,8 @@ e_scroll_frame_size_request (GtkWidget *widget, GtkRequisition *requisition)
 		else {
 			GtkWidgetAuxInfo *aux_info;
 
-			aux_info = gtk_object_get_data_by_id (GTK_OBJECT (bin->child),
-							      quark_aux_info);
+			aux_info = g_object_get_qdata (G_OBJECT (bin->child),
+						       quark_aux_info);
 			if (aux_info && aux_info->width > 0) {
 				requisition->width += aux_info->width;
 				extra_width = -1;
@@ -574,8 +586,8 @@ e_scroll_frame_size_request (GtkWidget *widget, GtkRequisition *requisition)
 		else {
 			GtkWidgetAuxInfo *aux_info;
 
-			aux_info = gtk_object_get_data_by_id (GTK_OBJECT (bin->child),
-							      quark_aux_info);
+			aux_info = g_object_get_qdata (G_OBJECT (bin->child),
+						       quark_aux_info);
 			if (aux_info && aux_info->height > 0) {
 				requisition->height += aux_info->height;
 				extra_height = -1;
@@ -681,8 +693,8 @@ e_scroll_frame_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 		xthickness = 0;
 		ythickness = 0;
 	} else {
-		xthickness = widget->style->klass->xthickness;
-		ythickness = widget->style->klass->ythickness;
+		xthickness = widget->style->xthickness;
+		ythickness = widget->style->ythickness;
 	}
 
 	if (bin->child && GTK_WIDGET_VISIBLE (bin->child)) {
@@ -782,7 +794,7 @@ e_scroll_frame_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 		gtk_widget_hide (priv->vsb);
 }
 
-/* Button press handler for the scroll framw diget */
+/* Button press handler for the scroll frame widget */
 static gint
 e_scroll_frame_button_press (GtkWidget *widget, GdkEventButton *event)
 {
@@ -795,11 +807,11 @@ e_scroll_frame_button_press (GtkWidget *widget, GdkEventButton *event)
 		GtkAdjustment *adj;
 		gfloat new_value;
 
-		gtk_object_get (GTK_OBJECT (widget),
-				(event->state & GDK_CONTROL_MASK) ?
-				 "hadjustment" : "vadjustment", 
-				 &adj,
-				NULL);
+		g_object_get (widget,
+			      (event->state & GDK_CONTROL_MASK) ?
+			      "hadjustment" : "vadjustment", 
+			      &adj,
+			      NULL);
 		new_value = adj->value + ((event->button == 4) ? 
 					  -adj->page_increment / 2: 
 					  adj->page_increment / 2);
@@ -969,7 +981,7 @@ e_scroll_frame_set_hadjustment (EScrollFrame *sf, GtkAdjustment *adj)
 	if (adj)
 		g_return_if_fail (GTK_IS_ADJUSTMENT (adj));
 	else
-		adj = GTK_ADJUSTMENT (gtk_object_new (GTK_TYPE_ADJUSTMENT, NULL));
+		adj = GTK_ADJUSTMENT (g_object_new (GTK_TYPE_ADJUSTMENT, NULL));
 
 	if (!priv->hsb) {
 		gtk_widget_push_composite_child ();
@@ -987,17 +999,19 @@ e_scroll_frame_set_hadjustment (EScrollFrame *sf, GtkAdjustment *adj)
 		if (old_adj == adj)
 			return;
 
-		gtk_signal_disconnect_by_func (GTK_OBJECT (old_adj),
-					       GTK_SIGNAL_FUNC (adjustment_changed),
-					       sf);
+		g_signal_handlers_disconnect_matched (old_adj,
+						      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
+						      0, 0, NULL,
+						      adjustment_changed,
+						      sf);
 		gtk_range_set_adjustment (GTK_RANGE (priv->hsb), adj);
 	}
 
 	adj = gtk_range_get_adjustment (GTK_RANGE (priv->hsb));
-	gtk_signal_connect (GTK_OBJECT (adj),
-			    "changed",
-			    GTK_SIGNAL_FUNC (adjustment_changed),
-			    sf);
+	g_signal_connect (adj,
+			  "changed",
+			  G_CALLBACK (adjustment_changed),
+			  sf);
 	adjustment_changed (adj, sf);
 
 	if (GTK_BIN (sf)->child)
@@ -1028,7 +1042,7 @@ e_scroll_frame_set_vadjustment (EScrollFrame *sf, GtkAdjustment *adj)
 	if (adj)
 		g_return_if_fail (GTK_IS_ADJUSTMENT (adj));
 	else
-		adj = GTK_ADJUSTMENT (gtk_object_new (GTK_TYPE_ADJUSTMENT, NULL));
+		adj = GTK_ADJUSTMENT (g_object_new (GTK_TYPE_ADJUSTMENT, NULL));
 
 	if (!priv->vsb) {
 		gtk_widget_push_composite_child ();
@@ -1046,17 +1060,19 @@ e_scroll_frame_set_vadjustment (EScrollFrame *sf, GtkAdjustment *adj)
 		if (old_adj == adj)
 			return;
 
-		gtk_signal_disconnect_by_func (GTK_OBJECT (old_adj),
-					       GTK_SIGNAL_FUNC (adjustment_changed),
-					       sf);
+		g_signal_handlers_disconnect_matched (old_adj,
+						      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
+						      0, 0, NULL,
+						      adjustment_changed,
+						      sf);
 		gtk_range_set_adjustment (GTK_RANGE (priv->vsb), adj);
 	}
 
 	adj = gtk_range_get_adjustment (GTK_RANGE (priv->vsb));
-	gtk_signal_connect (GTK_OBJECT (adj),
-			    "changed",
-			    GTK_SIGNAL_FUNC (adjustment_changed),
-			    sf);
+	g_signal_connect (adj,
+			  "changed",
+			  G_CALLBACK (adjustment_changed),
+			  sf);
 	adjustment_changed (adj, sf);
 
 	if (GTK_BIN (sf)->child)
@@ -1253,3 +1269,33 @@ e_scroll_frame_add_with_viewport (EScrollFrame *sf, GtkWidget *child)
 	gtk_widget_show (viewport);
 	gtk_container_add (GTK_CONTAINER (viewport), child);
 }
+
+void
+e_scroll_frame_get_policy (EScrollFrame *sf, GtkPolicyType *hsb_policy, GtkPolicyType *vsb_policy)
+{
+	g_return_if_fail (sf != NULL);
+	g_return_if_fail (E_IS_SCROLL_FRAME (sf));
+
+	if (hsb_policy)
+		*hsb_policy = ((ScrollFramePrivate *) sf->priv)->hsb_policy;
+	if (vsb_policy)
+		*vsb_policy = ((ScrollFramePrivate *) sf->priv)->vsb_policy;
+}
+
+gboolean
+e_scroll_frame_get_vscrollbar_visible (EScrollFrame *sf)
+{
+	g_return_val_if_fail (sf != NULL, FALSE);
+	g_return_val_if_fail (E_IS_SCROLL_FRAME (sf), FALSE);
+
+	return GTK_WIDGET_VISIBLE (((ScrollFramePrivate *) sf->priv)->vsb);
+}
+
+gboolean
+e_scroll_frame_get_hscrollbar_visible (EScrollFrame *sf)
+{
+	g_return_val_if_fail (sf != NULL, FALSE);
+	g_return_val_if_fail (E_IS_SCROLL_FRAME (sf), FALSE);
+
+	return GTK_WIDGET_VISIBLE (((ScrollFramePrivate *) sf->priv)->hsb);
+}
