@@ -31,6 +31,7 @@
 #ifndef __E_SCROLL_FRAME_H__
 #define __E_SCROLL_FRAME_H__
 
+#ifndef GAL_DISABLE_DEPRECATED
 
 #include <gdk/gdk.h>
 #include <gtk/gtkbin.h>
@@ -42,11 +43,11 @@ extern "C" {
 
 
 #define E_TYPE_SCROLL_FRAME            (e_scroll_frame_get_type ())
-#define E_SCROLL_FRAME(obj)            (GTK_CHECK_CAST ((obj), E_TYPE_SCROLL_FRAME, EScrollFrame))
-#define E_SCROLL_FRAME_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), E_TYPE_SCROLL_FRAME,	\
+#define E_SCROLL_FRAME(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), E_TYPE_SCROLL_FRAME, EScrollFrame))
+#define E_SCROLL_FRAME_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), E_TYPE_SCROLL_FRAME,	\
 					EScrollFrameClass))
-#define E_IS_SCROLL_FRAME(obj)         (GTK_CHECK_TYPE ((obj), E_TYPE_SCROLL_FRAME))
-#define E_IS_SCROLL_FRAME_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), E_TYPE_SCROLL_FRAME))
+#define E_IS_SCROLL_FRAME(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), E_TYPE_SCROLL_FRAME))
+#define E_IS_SCROLL_FRAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), E_TYPE_SCROLL_FRAME))
 
 
 typedef struct _EScrollFrame       EScrollFrame;
@@ -75,9 +76,14 @@ void e_scroll_frame_set_vadjustment (EScrollFrame *sf, GtkAdjustment *adj);
 GtkAdjustment *e_scroll_frame_get_hadjustment (EScrollFrame *sf);
 GtkAdjustment *e_scroll_frame_get_vadjustment (EScrollFrame *sf);
 
-void e_scroll_frame_set_policy (EScrollFrame *sf,
-				GtkPolicyType hsb_policy,
-				GtkPolicyType vsb_policy);
+void e_scroll_frame_set_policy (EScrollFrame  *sf,
+				GtkPolicyType  hsb_policy,
+				GtkPolicyType  vsb_policy);
+void e_scroll_frame_get_policy (EScrollFrame  *sf,
+				GtkPolicyType *hsb_policy,
+				GtkPolicyType *vsb_policy);
+gboolean e_scroll_frame_get_vscrollbar_visible (EScrollFrame *sf);
+gboolean e_scroll_frame_get_hscrollbar_visible (EScrollFrame *sf);
 
 void e_scroll_frame_set_placement (EScrollFrame *sf, GtkCornerType frame_placement);
 void e_scroll_frame_set_shadow_type (EScrollFrame *sf, GtkShadowType shadow_type);
@@ -89,5 +95,6 @@ void e_scroll_frame_add_with_viewport (EScrollFrame *sf, GtkWidget *child);
 }
 #endif /* __cplusplus */
 
+#endif /* GAL_DISABLE_DEPRECATED */
 
 #endif /* __E_SCROLL_FRAME_H__ */
